@@ -19,7 +19,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdint.h>
 #include "lsdvd.h"
+#include "ocode.h"
 
 static struct { char code[3];	char name[20];}
 language[] = {
@@ -76,6 +78,11 @@ double frames_per_s[4] = {-1.0, 25.00, -1.0, 29.97};
 
 char* program_name;
 
+//extern void operl_print(struct dvd_info *dvd_info);
+//extern void oxml_print(struct dvd_info *dvd_info);
+//extern void oruby_print(struct dvd_info *dvd_info);
+//extern void ohuman_print(struct dvd_info *dvd_info);
+
 int dvdtime2msec(dvd_time_t *dt)
 {
 	double fps = frames_per_s[(dt->frame_u & 0xc0) >> 6];
@@ -161,20 +168,59 @@ void version()
 void usage()
 {
 	version();
-	fprintf(stderr, "Usage: %s [options] [-t track_number] [dvd path] \n\nOptions:\n", program_name);
+	fprintf(stderr, "Usage: %s [options] [-t track_number] [dvd path] \n", program_name);
+	fprintf(stderr, "\n");
+	fprintf(stderr, "Options:\n");
 	fprintf(stderr, "\tExtra information:\n");
-	fprintf(stderr, "\t  -a audio streams\n\t  -d cells\n\t  -n angles\n\t  -c chapters\n\t  -s subpictures\n\t  -P palette\n");
-	fprintf(stderr, "\t  -v video\n\t  -x all information\n\n");
-	fprintf(stderr, "\tFormatting:\n\t  -Oh output as human readable (default)\n\t  -Op output as a Perl hash\n");
-	fprintf(stderr, "\t  -Or output as Ruby\n\t  -Ox output as XML\n\t  -p output as a Perl hash [deprecated]\n\n");
+	fprintf(stderr, "\t  -a audio streams\n");
+	fprintf(stderr, "\t  -d cells\n");
+	fprintf(stderr, "\t  -n angles\n");
+	fprintf(stderr, "\t  -c chapters\n");
+	fprintf(stderr, "\t  -s subpictures\n");
+	fprintf(stderr, "\t  -P palette\n");
+	fprintf(stderr, "\t  -v video\n");
+	fprintf(stderr, "\t  -x all information\n");
+	fprintf(stderr, "\n");
+	fprintf(stderr, "\tFormatting:\n");
+	fprintf(stderr, "\t  -Oh output as human readable (default)\n");
+	fprintf(stderr, "\t  -Op output as Perl\n");
+	fprintf(stderr, "\t  -Oy output as Python\n");
+	fprintf(stderr, "\t  -Or output as Ruby\n");
+	fprintf(stderr, "\t  -Ox output as XML\n");
+	//fprintf(stderr, "\t  -p output as Perl [deprecated]\n");
+	fprintf(stderr, "\n");
 	fprintf(stderr, "\tOther options:\n");
 	fprintf(stderr, "\t  -q quiet - no summary totals\n");
-	fprintf(stderr, "\t  -h this message\n\t  -V version information\n\n");
+	fprintf(stderr, "\t  -h this message\n");
+	fprintf(stderr, "\t  -V version information\n");
+	fprintf(stderr, "\n");
 }
 
 int opt_a=0, opt_c=0, opt_n=0, opt_p=0, opt_q=0, opt_s=0, opt_t=0, opt_v=0, opt_x=0, opt_d=0, opt_P=0;
 char opt_O='h';
 
+char output_option(char *arg)
+{
+	if (strlen(arg) == 1) {
+		return arg[0];
+	} else if (strcmp(arg, "perl") == 0) {
+		return 'p';
+	} else if (strcmp(arg, "python") == 0) {
+		return 'y';
+	} else if (strcmp(arg, "ruby") == 0) {
+		return 'r';
+	} else if (strcmp(arg, "ruby2") == 0) {
+		return '2';
+	} else if (strcmp(arg, "xml") == 0) {
+		return 'x';
+	} else if (strcmp(arg, "human") == 0) {
+		return 'h';
+	} else {
+		return '\0';
+	}
+}
+										
+
 int main(int argc, char *argv[])
 {
 	char title[33];
@@ -210,7 +256,7 @@ int main(int argc, char *argv[])
 		case 'p':	opt_p = 1;		break;
 		case 'P':	opt_P = 1;		break;
 		case 't':	opt_t = atoi(optarg);	break;
-		case 'O':	opt_O = optarg[0];	break;
+		case 'O':	opt_O = output_option(optarg);	break;
 		case 'v':	opt_v = 1;		break;
 		case 'x':	opt_x = 1;
 				opt_a = 1;
@@ -320,7 +366,7 @@ int main(int argc, char *argv[])
 		// PALETTE
 		if (opt_P) {
 			dvd_info.titles[j].palette = malloc(16 * sizeof(int));
-        	        for (i=1; i < 18; i++) { dvd_info.titles[j].palette[i] = pgc->palette[i]; }
+        	        for (i=1; i < 16; i++) { dvd_info.titles[j].palette[i] = pgc->palette[i]; }
 		} else {
 			dvd_info.titles[j].palette = NULL;
 		}
@@ -428,13 +474,27 @@ int main(int argc, char *argv[])
 	if (! opt_t) { dvd_info.longest_track = max_track; }
 
 	if (opt_p) {
-		operl_print(&dvd_info);
+		ocode_print(&perl_syntax, &dvd_info);
 	} else {
 		switch(opt_O) {
-			case 'p':	operl_print(&dvd_info);		break;
-			case 'x':	oxml_print(&dvd_info);		break;
-			case 'r':	oruby_print(&dvd_info);		break;
-			default :	ohuman_print(&dvd_info);	break;
+			case 'p':	
+				ocode_print(&perl_syntax, &dvd_info);		
+				break;
+                        case 'y':
+			       ocode_print(&python_syntax, &dvd_info);           
+				break;
+			case 'x':
+				oxml_print(&dvd_info);		
+				break;
+			case 'r':
+				ocode_print(&ruby_syntax, &dvd_info);		
+				break;
+                        case 'd':
+				ocode_print(&debug_syntax, &dvd_info);           
+				break;
+			default :
+				ohuman_print(&dvd_info);	
+				break;
 		}
 	}
 
