@@ -74,6 +74,9 @@ struct dvd_info {
 	int longest_track;
 };
 
-
+void ohuman_print(struct dvd_info *dvd_info);
+void operl_print(struct dvd_info *dvd_info);
+void oruby_print(struct dvd_info *dvd_info);
+void oxml_print(struct dvd_info *dvd_info);
 
 #endif /* _LSDVD_H_ */
